
import os
from grok_client import GrokClient
from grok_payload import GrokPayload
from grok_cookies import GrokCookies

if os.path.exists(".env"):
    from dotenv import load_dotenv
    load_dotenv()



if __name__ == "__main__":


    grok_client = GrokClient(
        cookies=GrokCookies(
            x_challenge=os.getenv("X_CHALLENGE"),
            x_anonuserid=os.getenv("X_ANONUSERID"),
            x_signature=os.getenv("X_SIGNATURE"),
            sso=os.getenv("SSO"),
            sso_rw=os.getenv("SSO_RW")
        )
    )
    new = grok_client.new(GrokPayload(message="My name is Enciyo."))

    for r in new:
        if r.response and r.response.modelResponse:
            print(r.response.modelResponse.message)
        if r.conversation and r.conversation.conversationId:
            conversation_id = r.conversation.conversationId

    responses = grok_client.responses(
        conversation_id=conversation_id,
        data=GrokPayload(message="What's my name?"),
    )

    for r in responses:
        if r.response and r.response.modelResponse:
            print(r.response.modelResponse.message)



from dataclasses import dataclass

@dataclass
class GrokPayload:
    temporary: bool = False
    modelName: str = "grok-2"
    message: str = ""
    fileAttachments: list = None
    imageAttachments: list = None
    disableSearch: bool = False
    enableImageGeneration: bool = True
    returnImageBytes: bool = False
    returnRawGrokInXaiRequest: bool = False
    enableImageStreaming: bool = True
    imageGenerationCount: int = 2
    forceConcise: bool = False
    toolOverrides: dict = None
    enableSideBySide: bool = True
    isPreset: bool = False
    sendFinalMetadata: bool = True
    customInstructions: str = ""
    deepsearchPreset: str = ""
    isReasoning: bool = False





from dataclasses import dataclass

from dataclasses import dataclass
from typing import List, Optional, Dict, Any
from datetime import datetime
import json
import re


def parse_datetime(dt_str: str) -> datetime:
    match = re.match(r"(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2})\.(\d+)Z", dt_str)
    if match:
        base_time_str = match.group(1)
        microseconds_str = match.group(2)

        microseconds = int(microseconds_str)
        microseconds = round(microseconds / (10 ** (len(microseconds_str) - 6)))

        datetime_obj = datetime.strptime(base_time_str, "%Y-%m-%dT%H:%M:%S")
        datetime_obj = datetime_obj.replace(microsecond=microseconds)
        return datetime_obj
    else:
        raise ValueError(f"Geçersiz tarih formatı: {dt_str}")

@dataclass
class BaseResponse:
    responseId: str
    message: str
    sender: str
    createTime: datetime
    manual: bool
    partial: bool
    shared: bool
    query: str
    queryType: str
    webSearchResults: List[str]
    xpostIds: List[str]
    xposts: List[str]
    generatedImageUrls: List[str]
    imageAttachments: List[str]
    fileAttachments: List[str]
    cardAttachmentsJson: List[str]
    fileUris: List[str]
    fileAttachmentsMetadata: List[str]
    isControl: bool
    steps: List[str]
    imageEditUris: List[str]
    mediaTypes: List[str]
    webpageUrls: List[str]

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'BaseResponse':
        return cls(
            responseId=data["responseId"],
            message=data["message"],
            sender=data["sender"],
            createTime=parse_datetime(data["createTime"]),
            manual=data["manual"],
            partial=data["partial"],
            shared=data["shared"],
            query=data["query"],
            queryType=data["queryType"],
            webSearchResults=data["webSearchResults"],
            xpostIds=data["xpostIds"],
            xposts=data["xposts"],
            generatedImageUrls=data["generatedImageUrls"],
            imageAttachments=data["imageAttachments"],
            fileAttachments=data["fileAttachments"],
            cardAttachmentsJson=data["cardAttachmentsJson"],
            fileUris=data["fileUris"],
            fileAttachmentsMetadata=data["fileAttachmentsMetadata"],
            isControl=data["isControl"],
            steps=data["steps"],
            imageEditUris=data["imageEditUris"],
            mediaTypes=data["mediaTypes"],
            webpageUrls=data["webpageUrls"]
        )

@dataclass
class UserResponse(BaseResponse):
    pass

@dataclass
class ModelResponse(BaseResponse):
    parentResponseId: str
    metadata: Dict[str, str]

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'ModelResponse':
        base = BaseResponse.from_dict(data)
        return cls(
            **vars(base),
            parentResponseId=data["parentResponseId"],
            metadata=data["metadata"]
        )

@dataclass
class Conversation:
    conversationId: str
    title: str
    starred: bool
    createTime: datetime
    modifyTime: datetime
    systemPromptName: str
    temporary: bool
    mediaTypes: List[str]

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'Conversation':
        return cls(
            conversationId=data["conversationId"],
            title=data["title"],
            starred=data["starred"],
            createTime=parse_datetime(data["createTime"]),
            modifyTime=parse_datetime(data["modifyTime"]),
            systemPromptName=data["systemPromptName"],
            temporary=data["temporary"],
            mediaTypes=data["mediaTypes"]
        )

@dataclass
class TokenResponse:
    token: str
    isThinking: bool
    isSoftStop: bool
    responseId: str

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'TokenResponse':
        return cls(
            token=data["token"],
            isThinking=data["isThinking"],
            isSoftStop=data["isSoftStop"],
            responseId=data["responseId"]
        )

@dataclass
class Response:
    userResponse: Optional[UserResponse] = None
    modelResponse: Optional[ModelResponse] = None
    token: Optional[TokenResponse] = None
    isThinking: bool = False
    isSoftStop: bool = False
    responseId: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'Response':
        user_response = UserResponse.from_dict(data["userResponse"]) if "userResponse" in data else None
        model_response = ModelResponse.from_dict(data["modelResponse"]) if "modelResponse" in data else None
        token = TokenResponse.from_dict(data) if "token" in data else None
        return cls(
            userResponse=user_response,
            modelResponse=model_response,
            token=token,
            isThinking=data.get("isThinking", False),
            isSoftStop=data.get("isSoftStop", False),
            responseId=data.get("responseId", "")
        )


@dataclass
class ConversationResult:
    conversation: Optional[Conversation] = None
    response: Optional[Response] = None
    title: Optional[Dict[str, str]] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'ConversationResult':
        result = data.get("result", {})
        if "conversation" in result:
            return cls(conversation=Conversation.from_dict(result["conversation"]))
        elif "response" in result:
            return cls(response=Response.from_dict(result["response"]))
        elif "title" in result:
            return cls(title=result["title"])
        elif "modelResponse" in result:
            return cls(response= Response(
                modelResponse=ModelResponse.from_dict(result["modelResponse"]),
            ))
        return cls()

def parse_multi_json(text: str) -> List[ConversationResult]:
    json_objects = re.findall(r'\{.*?\}(?=\s*\{|$)', text, re.DOTALL)
    results = []

    for json_str in json_objects:
        try:
            data = json.loads(json_str.strip())
            result = ConversationResult.from_dict(data)
            results.append(result)
        except json.JSONDecodeError as e:
            print(f"JSON ayrıştırma hatası: {e} - Geçersiz JSON: {json_str}")

    return results



import requests
from grok_unoffical_api._headers_manager import HeadersManager
from grok_unoffical_api.grok_payload import GrokPayload
from grok_unoffical_api.grok_cookies import GrokCookies
from grok_unoffical_api.grok_response import parse_multi_json


class GrokClient:
    def __init__(self,headers_manager: HeadersManager = HeadersManager(),cookies: GrokCookies = None):
        self.headers = headers_manager.get_headers()
        self.cookies = cookies.to_dict()

    def new(self,messages : GrokPayload = None):
        response = requests.post(
            "https://grok.com/rest/app-chat/conversations/new",
            headers=self.headers,
            cookies=self.cookies,
            json=messages.__dict__
        )
        if response.status_code == 200:
            return parse_multi_json(response.text)
        else:
            raise Exception(f"Error: {response.status_code} - {response.text}")


    def responses(self,conversation_id:str,data : GrokPayload = None):
        response = requests.post(
            f"https://grok.com/rest/app-chat/conversations/{conversation_id}/responses",
            headers=self.headers,
            cookies=self.cookies,
            json= data.__dict__
        )
        if response.status_code == 200:
            return parse_multi_json(response.text)
        else:
            raise Exception(f"Error: {response.status_code} - {response.text}")






from dataclasses import dataclass


@dataclass
class GrokCookies:
    x_anonuserid: str
    x_challenge: str
    x_signature: str
    sso_rw: str
    sso:str

    def to_dict(self) -> dict:
        return {
            "x-anonuserid": self.x_anonuserid,
            "x-challenge": self.x_challenge,
            "x-signature": self.x_signature,
            "sso-rw": self.sso_rw,
            "sso": self.sso
        }







class HeadersManager:
    def get_headers(self):
        return {
            "accept": "*/*",
            "accept-language": "en-GB,en;q=0.9",
            "content-type": "application/json",
            "origin": "https://grok.com",
            "priority": "u=1, i",
            "referer": "https://grok.com/",
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "sec-gpc": "1",
            f"User-Agent": "PostmanRuntime/7.43.3",
        }



from setuptools import setup, find_packages

setup(
    name='grok_unoffical_api',
    version='0.1.1',
    description='Unofficial Grok API',
    author='Mustafa K',
    author_email='ben@mustafakilic.dev',
    url= "https://github.com/enciyo/grok-unofficial-api",
    packages=find_packages(),
    install_requires=[
        'requests',
        'python-dotenv'
    ],
)
